/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Index Controller - IndexSearchService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.unige.solidify.service;

import java.util.HashMap;
import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;

import ch.unige.solidify.IndexConstants;
import ch.unige.solidify.model.index.IndexMetadata;
import ch.unige.solidify.rest.FacetPage;
import ch.unige.solidify.rest.FacetRequest;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.rest.RestCollectionPage;
import ch.unige.solidify.service.rest.abstractservice.IndexMetadataRemoteResourceService;

public abstract class IndexSearchService<I, T extends IndexMetadata> extends IndexResourceService<I, T> {

  private final IndexMetadataRemoteResourceService<I, T> indexMetadataRemoteResourceService;

  protected IndexSearchService(IndexMetadataRemoteResourceService<I, T> indexMetadataRemoteResourceService) {
    this.indexMetadataRemoteResourceService = indexMetadataRemoteResourceService;
  }

  @Override
  public List<T> findAll(I indexName, T search) {
    HashMap<String, Object> parameters = new HashMap<>();
    parameters.put(IndexConstants.SEARCH_QUERY, "*");
    final RestCollection<T> result = this.indexMetadataRemoteResourceService.getIndexMetadataList(
            indexName,
            parameters,
            PageRequest.of(0, RestCollectionPage.MAX_SIZE_PAGE));
    return result.getData();
  }

  @Override
  public Page<T> findAll(I indexName, T search, Pageable pageable) {
    HashMap<String, Object> parameters = new HashMap<>();
    parameters.put(IndexConstants.SEARCH_QUERY, "*");
    final RestCollection<T> result = this.indexMetadataRemoteResourceService.getIndexMetadataList(
            indexName,
            parameters,
            pageable);
    return new PageImpl<>(result.getData(), pageable, result.getPage().getTotalItems());
  }

  @Override
  public T findOne(I indexName, String id) {
    return this.indexMetadataRemoteResourceService.getIndexMetadata(indexName, id);
  }

  @Override
  public FacetPage<T> search(I indexName, String query, String conditions, List<FacetRequest> facetRequests,
          Pageable pageable) {
    final RestCollection<T> result = this.indexMetadataRemoteResourceService.getIndexMetadataListWithConditions(
            indexName,
            query,
            conditions,
            pageable);
    return new FacetPage<>(result.getData(), result.getFacets(), pageable, result.getPage().getTotalItems());
  }

  @Override
  public T save(I indexName, T t) {
    throw new UnsupportedOperationException();
  }

  @Override
  public T update(I indexName, T t) {
    throw new UnsupportedOperationException();
  }

  @Override
  public boolean delete(I indexName, T t) {
    throw new UnsupportedOperationException();
  }

  @Override
  public void deleteList(I indexName, List<String> ids) {
    throw new UnsupportedOperationException();
  }
}
